﻿using System;
using System.Collections.Generic;
using System.Text;

namespace WinGetOpt {

	//***************************************************************************
	/// <summary>	OptionParserによって解析された、パラメータの情報.
	///				解析前のパラメータ情報と、パラメータ定義は、それぞれRawArgs,
	///				Optionsで指定可能
	/// </summary>
	/// <author>	aka(nanoappli.com)</author>
	//***************************************************************************
	class CommandLine {
		/// <summary>解析前パラメータの生データ</summary>
		public String[] RawArgs{ set; get; }

		/// <summary>パラメータ解析に使用した定義情報</summary>
		public Options  Options{ set; get; }

		/// <summary>解析後パラメータの情報</summary>
		public Dictionary< String, List<String> > ParsedParam = new Dictionary<string,List<string>>();


		//***************************************************************************
		/// <summary> 指定されたキーに対応するパラメータ値を取得する
		/// </summary>
		/// <param name="key">		キー</param>
		/// <returns>				値</returns>
		//***************************************************************************
		public String GetOptionValue( string key ) {
			if ( !ParsedParam.ContainsKey( key ) ) {
				return null;
			}
			return ParsedParam[ key ][0];
		}


		//***************************************************************************
		/// <summary> 指定されたキーに対応するパラメータ値を全て取得する
		/// </summary>
		/// <param name="key">		キー</param>
		/// <returns>				値の一覧</returns>
		//***************************************************************************
		public String[] GetOptionValues( string key ) {
			if ( !ParsedParam.ContainsKey( key ) ) {
				return null;
			}
			return ParsedParam[ key ].ToArray();
		}


		//***************************************************************************
		/// <summary> 指定されたキーがパラメータ指定されているか取得する
		/// </summary>
		/// <param name="key">		キー</param>
		/// <returns>				true:指定されている, false:未指定</returns>
		//***************************************************************************
		public bool HasOption( string key ) {
			return ParsedParam.ContainsKey( key );
		}	
	

		//***************************************************************************
		/// <summary> 指定されたパラメータを、解析済みとして追加する
		/// </summary>
		/// <param name="key">		キー</param>
		/// <param name="value">	値</param>
		//***************************************************************************
		public void addParam( string key, string value ) {
			if ( ParsedParam.ContainsKey( key ) ) {
				//-------------------------------------
				// 既にキーが存在する -> 値一覧に追加
				//-------------------------------------
				ParsedParam[ key ].Add( value );
				return;
			}

			//--------------------------------------------
			// 初出のキー -> valueを入れる器を作って追加
			//--------------------------------------------
			List<String> valueList = new List<string>();
			valueList.Add( value );
			ParsedParam[key] = valueList;
			return;
		}


		//***************************************************************************
		/// <summary> このオブジェクトの文字列表現を返す
		///			  例 param={key:xxx value:xxx,}, {key:xxx value:{xxx,xxx}}, ... ,}
		/// </summary>
		/// <returns>内部状態を文字で表現したもの</returns>
		//***************************************************************************
		public override String ToString() {
			StringBuilder outStr = new StringBuilder();

			outStr.Append( "param={" );
			foreach ( String key in ParsedParam.Keys ) {
				// キーを出力
				outStr.Append( "{key:" + key + " " );

				// 値を出力
				if ( ParsedParam[ key ].Count == 1 ) {
					// 値が1個だけの場合
					outStr.Append( "value:" + ParsedParam[ key ][0] + "}, " );
				} else {
					// 値が複数の場合
					outStr.Append( "value:{" );
					foreach ( string value in ParsedParam[ key ] ) {
						outStr.Append( value + "," );				
					}
					outStr.Append( "}," );
				}
			}
			return outStr.ToString();
		}


	}
}

